import logging
from typing import List, Union

import numpy as np
import treefiles as tf

from gcma.io.BaseIO import BaseIO, TValue, Bases
from gcma.io.utils import r


class Feature(BaseIO):
    def __init__(self, *args, ref: TValue, error: TValue = None, **kwargs):
        super().__init__(*args, **kwargs)
        self.ref = ref
        self._error = error

        self.registered.add("ref")
        self.registered.add("error")

    @property
    def error(self):
        if self._error is None:
            if self.value is not None:
                return self.error2
        else:
            return self._error

    @error.setter
    def error(self, value: TValue):
        self._error = value

    @property
    def error2(self) -> TValue:
        """
        Return (x - ref) ** 2
        """
        return np.square(self.value - self.ref)

    @property
    def error_rel(self) -> TValue:
        """
        Return relative error (x - ref) / ref
        If ref=0, returns self.error2
        """
        if self.ref == 0:
            return self.error2
        return (self.value - self.ref) / self.ref


class Features(Bases):
    inner_class = Feature

    @property
    def table(self) -> tf.Table:
        header = ["Name", "Value", "Ref", "Error"]
        data = []
        for x in self.values():
            # data.append([x.name, r(x.value), r(x.ref), f"{r(x * 100)}%"])
            data.append([x.name, r(x.value), r(x.ref), r(x.error)])
        return tf.Table(header=header, rows=data)


TFeatures = Union[List[Feature], Features]

log = logging.getLogger(__name__)
