import logging
from typing import Tuple, List, Union

import treefiles as tf

from gcma.io.BaseIO import BaseIO, TValue, Bases
from gcma.io.utils import r

_Bounds = Tuple[TValue, TValue]


class Param(BaseIO):
    def __init__(self, *args, initial_value: TValue, bounds: _Bounds = None, **kwargs):
        super().__init__(*args, **kwargs)
        self.initial_value = initial_value
        self.bounds = bounds

        self.registered.add("initial_value")
        self.registered.add("bounds")


class Params(Bases):
    inner_class = Param

    @property
    def table(self) -> tf.Table:
        header = ["Name", "Baseline", "Value", "Bounds"]
        data = []
        for x in self.values():
            data.append([x.name, x.initial_value, r(x.value), x.bounds])
        return tf.Table(header=header, rows=data)


TParams = Union[List[Param], Params]

log = logging.getLogger(__name__)
